@@ -1,5 +1,6 @@
 .py_email <- NULL
 .py_codecs <- NULL
+.py_nul_str <- NULL
 
 .onLoad <- function(libname, pkgname) {
     rapv <- Sys.getenv("RETICULATE_AUTOCREATE_PACKAGE_VENV", "")
@@ -8,4 +9,6 @@
     Sys.setenv("RETICULATE_AUTOCREATE_PACKAGE_VENV" = "false")
     .py_email <<- import("email", delay_load = TRUE)
     .py_codecs <<- import("codecs", delay_load = TRUE)
+    .py_nul_str <<-
+        import_builtins(convert = FALSE, delay_load = TRUE)$chr(0L)
 }
